/*
 *  linux/arch/arm/mach-ebsa110/leds.c
 *
 *  Copyright (C) 1998 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *  EBSA-110 LED control routines.  We use the led as follows:
 *
 *   - Red - toggles state every 50 timer interrupts
 */
#include <linux/kernel.h>
#include <linux/spinlock.h>
#include <linux/init.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/leds.h>
#include <asm/system.h>
#include <asm/mach-types.h>

static  spinlock_t 	leds_lock;
static  u32   		led_val = 0;
static	u32		led_count = 0;
extern void mvBoardDebug7Seg(u32);


int blink_pwr_led=0;
int on_off_ant=2;
int enable_RF=0; 


#include "mvGppRegs.h"
#include "mvGpp.h"
extern int mvCheckAntBtnStatus();
static int radio_status=0;
static u32 ant_gpp_group = (MV_GPP19|MV_GPP20|MV_GPP21|MV_GPP22|MV_GPP23|MV_GPP24|MV_GPP25|MV_GPP26);
#define WLAN_LED_BLINK_RATE     2
static void blink_rf_led();

static void mv_leds_event(led_event_t ledevt)
{
	unsigned long flags;
    static int status=-1, last_status=-1;
    static int blink_pwr_led_on_off=0;
 
	spin_lock_irqsave(&leds_lock, flags);

	switch(ledevt) {
	case led_timer:
	

	    if ( blink_pwr_led == 1 )
	    {
            if (blink_pwr_led_on_off==1)
            {
                mvGppValueSet(0, MV_GPP5, MV_GPP5);
                blink_pwr_led_on_off=0;
            }
            else
            {
                mvGppValueSet(0, MV_GPP5, 0);
                blink_pwr_led_on_off=1;
            }
            
            break;
	    }




        if ( enable_RF == 0 )
        {
            status = 0;
        }
        else
        {
            status = mvCheckAntBtnStatus();
        }

        radio_status = status;

        if ( last_status == status )
        {
            break;
        }
        
        if ( 1 == status )
        {
	     printk("Wireless atthena LED button turned on\n");
        	mvGppValueSet(0, ant_gpp_group, 0);
        }
        else
        {
	        printk("Wireless atthena LED button turned off\n");
        	mvGppValueSet(0, ant_gpp_group, ant_gpp_group);
        }
        
        last_status = status;

		break;
    case 1000:
    
        if ( enable_RF == 0 )
        {
            break;
        }
        
        blink_rf_led();
    
        break;
	default:
			break;
	}

	spin_unlock_irqrestore(&leds_lock, flags);
}


#include <linux/netdevice.h>
#include <linux/netlink.h>
#include <linux/delay.h>
#define WLAN_IF_NAME            "ap0"

int current_mode=0;
static unsigned int led_back=1;
static unsigned int led_forth=8;

#define LED_ALL     MV_GPP19|MV_GPP20|MV_GPP21|MV_GPP22|MV_GPP23|MV_GPP24|MV_GPP25|MV_GPP26
#define RX1_RSSI    MV_GPP25|MV_GPP22|MV_GPP19
#define RX2_RSSI    MV_GPP26|MV_GPP23|MV_GPP20
#define RX3_RSSI    MV_GPP24|MV_GPP21

static void back_and_forth_blink(int num)
{

    switch (num)
    {
        case 1:
            mvGppValueSet(0, MV_GPP19, 0);
            break;
        case 2:
            mvGppValueSet(0, MV_GPP20, 0);
            break;
        case 3:
            mvGppValueSet(0, MV_GPP21, 0);
            break;
        case 4:
            mvGppValueSet(0, MV_GPP22, 0);
            break;
        case 5:
            mvGppValueSet(0, MV_GPP23, 0);
            break;
        case 6:
            mvGppValueSet(0, MV_GPP24, 0);
            break;
        case 7:
            mvGppValueSet(0, MV_GPP25, 0);
            break;
        case 8:
            mvGppValueSet(0, MV_GPP26, 0);
            break;
    }
}

static int check_traffic()
{
    struct net_device *wlan_dev = NULL;
    static int interrupt_count = 0;
    static int pkt_cnt_old_rx = 0;
    static int pkt_cnt_old_tx = 0;
    struct ifreq ifr;
    int ret=0;

    wlan_dev = dev_get_by_name(WLAN_IF_NAME);
    if (wlan_dev) 
    {

        struct net_device_stats *wlan_dev_stats = NULL;
        int pkt_cnt_rx, pkt_cnt_tx;


        wlan_dev_stats = wlan_dev->get_stats(wlan_dev);
        if (wlan_dev_stats) 
        {

            pkt_cnt_rx = wlan_dev_stats->rx_packets;
            if (pkt_cnt_rx != pkt_cnt_old_rx) 
            {
                pkt_cnt_old_rx = pkt_cnt_rx;
                ret=1;
            }
            

            if ( ret == 0 )
            {
                pkt_cnt_tx = wlan_dev_stats->tx_packets;
                if (pkt_cnt_tx != pkt_cnt_old_tx) 
                {
                    pkt_cnt_old_tx = pkt_cnt_tx;
                    ret=1;
                }
            }
        }

        dev_put(wlan_dev);
    }

    return ret;
}

static int blink_ant_led()
{
    static int all_on_off_timer=0;
    static int on_off=0;

    mvGppValueSet(0, LED_ALL, LED_ALL);
    
    if ( 0 == current_mode )
    {

        int i=0;
       
        if (led_back < 8)
            i = ++led_back;
        else if ( led_forth > 1 )
            i = --led_forth;
        else
        {
            i=1;
            led_back=1;
            led_forth=8;
        }

        back_and_forth_blink(i);
    }
    else
    {
        if ( check_traffic() == 0 )
        {
            if ( all_on_off_timer == 0 )
            {
                all_on_off_timer++;
            }
            else if ( all_on_off_timer == 8 ) 
            {
                all_on_off_timer=0;
                mvGppValueSet(0, LED_ALL, 0);
            }
            else
                all_on_off_timer++;
        }
        else
        {
            all_on_off_timer=0;

            if ( on_off == 1 )
            {
                switch (current_mode)
                {
                    case 1:
                        mvGppValueSet(0, RX1_RSSI, 0);
                        break;
                    case 2:
                        mvGppValueSet(0, RX2_RSSI, 0);
                        break;
                    case 3:
                        mvGppValueSet(0, RX3_RSSI, 0);
                        break;
                }

                on_off=0;
            }
            else
                on_off=1;
        }
    }
    
    return 0;
}

static int wlan_led_on_off(int on_off)
{
    if ( on_off == 1 )
        mvGppValueSet(0 ,MV_GPP11,MV_GPP11);
    else
        mvGppValueSet(0 ,MV_GPP11,0);

    return 0;
}

static void blink_rf_led()
{
    struct net_device *wlan_dev = NULL;
    static int interrupt_count = 0;
    static int pkt_cnt_old = 0;
    static int led_state = 0;

    interrupt_count++;
    if (interrupt_count == WLAN_LED_BLINK_RATE/2)
        wlan_led_on_off(led_state);
    if (interrupt_count < WLAN_LED_BLINK_RATE)
        return;
        
    interrupt_count = 0;
    
    wlan_dev = dev_get_by_name(WLAN_IF_NAME);
    if (wlan_dev) {
        if (wlan_dev->flags & IFF_UP) {
            /* Interface is up and running, check device stats */
            struct net_device_stats *wlan_dev_stats = NULL;
            int    pkt_cnt;

            led_state = 1;

            /* Check total packet count, if different from before, blink LED. */
            wlan_dev_stats = wlan_dev->get_stats(wlan_dev);
            if (wlan_dev_stats) {
                pkt_cnt = wlan_dev_stats->tx_packets + wlan_dev_stats->rx_packets;
                if (pkt_cnt != pkt_cnt_old) {
                    pkt_cnt_old = pkt_cnt;
                    wlan_led_on_off(0);
                }
            }
        } else {
            /* Interface is down, turn LED off */
            led_state = 0;
        }
        dev_put(wlan_dev);
    } else {
        /* Interface not ready, turn LED off */
        led_state = 0;
    }
}

static void mv_leds_event2(led_event_t ledevt)
{
	unsigned long flags;
	static int status=-1, last_status=-1;
	spin_lock_irqsave(&leds_lock, flags);

    /* Check radio status */
    if ( radio_status == 0 )
        return;

    switch (on_off_ant)
    {
        case 0: /*off*/
            mvGppValueSet(0, ant_gpp_group, ant_gpp_group);
            return;
        case 1: /*on*/
            mvGppValueSet(0, ant_gpp_group, 0);
            return;
    }



    blink_ant_led();


    spin_unlock_irqrestore(&leds_lock, flags);
}


static int __init leds_init(void)
{
#ifdef CONFIG_LEDS
	leds_event = mv_leds_event;
	leds_event2 = mv_leds_event2;
#endif
	return 0;
}

__initcall(leds_init);
